import flask
import json
import pytest
from unittest.mock import patch, mock_open, call
from werkzeug.exceptions import HTTPException
from api import create_app
from endpoints import receive
from conftest import get_auth_header


@pytest.mark.usefixtures(
    "mock_clients", "mock_client_credentials", "mock_read_from_queue_return"
)
def test_get_receive(
    mock_clients, mock_client_credentials, mock_read_from_queue_return
):
    app = create_app()
    with patch(
        "builtins.open", mock_open(read_data=json.dumps(mock_clients))
    ) as mock_file_open, patch.object(
        receive, "read_from_queue"
    ) as mock_read_from_queue, app.test_client() as app_test_client:
        mock_read_from_queue.return_value = mock_read_from_queue_return
        auth_header = get_auth_header(app_test_client, mock_client_credentials)
        response = app_test_client.get("/receive", headers=auth_header)
        assert response.status_code == 200
        mock_read_from_queue.assert_called_once_with(
            queue_name="client-1-inbox", max_msgs=10
        )
        assert response.json == mock_read_from_queue_return


@pytest.mark.usefixtures(
    "mock_clients", "mock_client_credentials", "mock_read_from_queue_return"
)
def test_get_receive_max_messages(
    mock_clients, mock_client_credentials, mock_read_from_queue_return
):
    app = create_app()
    with patch(
        "builtins.open", mock_open(read_data=json.dumps(mock_clients))
    ) as mock_file_open, patch.object(
        receive, "read_from_queue"
    ) as mock_read_from_queue, app.test_client() as app_test_client:
        mock_read_from_queue.return_value = mock_read_from_queue_return
        auth_header = get_auth_header(app_test_client, mock_client_credentials)
        max_messages = 2
        response = app_test_client.get(
            "/receive", query_string={"max_messages": max_messages}, headers=auth_header
        )
        assert response.status_code == 200
        mock_read_from_queue.assert_called_once_with(
            queue_name="client-1-inbox", max_msgs=max_messages
        )
        assert response.json == mock_read_from_queue_return


@pytest.mark.usefixtures("mock_clients", "mock_read_from_queue_return")
def test_get_receive_no_token(mock_clients, mock_read_from_queue_return):
    app = create_app()
    with patch(
        "builtins.open", mock_open(read_data=json.dumps(mock_clients))
    ) as mock_file_open, patch.object(
        receive, "read_from_queue"
    ) as mock_read_from_queue, app.test_client() as app_test_client:
        mock_read_from_queue.return_value = mock_read_from_queue_return
        response = app_test_client.get("/receive")
        assert response.status_code == 403
        mock_read_from_queue.assert_not_called()


@pytest.mark.usefixtures("mock_clients", "mock_read_from_queue_return")
def test_get_receive_invalid_token(mock_clients, mock_read_from_queue_return):
    app = create_app()
    with patch(
        "builtins.open", mock_open(read_data=json.dumps(mock_clients))
    ) as mock_file_open, patch.object(
        receive, "read_from_queue"
    ) as mock_read_from_queue, app.test_client() as app_test_client:
        mock_read_from_queue.return_value = mock_read_from_queue_return
        auth_header = {"Authorization": "made-up-token"}
        response = app_test_client.get("/receive", headers=auth_header)
        assert response.status_code == 403
        mock_read_from_queue.assert_not_called()
