import json
import os
import pytest
import re
from unittest.mock import patch, mock_open, call
from models.client_model import ClientModel


@pytest.mark.usefixtures("mock_clients")
def test_init_with_clients(mock_clients):
    # File is read on __init__
    with patch(
        "builtins.open", mock_open(read_data=json.dumps(mock_clients))
    ) as mock_file_open:
        clients_file = ClientModel()
        assert len(clients_file.clients.keys()) == 2
        assert "client-1" in clients_file.clients.keys()


def test_init_no_clients_file():
    # Handles FileNotFound on __init__
    with patch("builtins.open", side_effect=FileNotFoundError()) as mock_file_open:
        clients_file = ClientModel()
        assert len(clients_file.clients.keys()) == 0
        assert "client-1" not in clients_file.clients.keys()


@pytest.mark.usefixtures("mock_clients")
def test_get_unmodified(mock_clients):
    # If modtime is unchanged file is not read
    with patch(
        "builtins.open", mock_open(read_data=json.dumps(mock_clients))
    ) as mock_file_open, patch("os.path.getmtime", return_value=3) as mock_getmtime:
        clients_file = ClientModel()
        clients = clients_file.get()
        assert mock_file_open.call_count == 1


@pytest.mark.usefixtures("mock_clients")
def test_get_modified(mock_clients):
    # If modtime has changed file is re-read
    with patch(
        "builtins.open", mock_open(read_data=json.dumps(mock_clients))
    ) as mock_file_open, patch("os.path.getmtime", return_value=3) as mock_getmtime:
        clients_file = ClientModel()
        mock_getmtime.return_value = 4
        clients = clients_file.get()
        assert mock_file_open.call_count == 2


@pytest.mark.usefixtures("mock_clients")
def test_find_existing(mock_clients):
    with patch(
        "builtins.open", mock_open(read_data=json.dumps(mock_clients))
    ) as mock_file_open:
        clients_file = ClientModel()
        client = clients_file.find("client-1")
        assert client is not None
        assert client.get("subscription") == "soar.#"


@pytest.mark.usefixtures("mock_clients")
def test_find_nonexisting(mock_clients):
    with patch(
        "builtins.open", mock_open(read_data=json.dumps(mock_clients))
    ) as mock_file_open:
        clients_file = ClientModel()
        client = clients_file.find("client-3")
        assert client is None


@pytest.mark.usefixtures("mock_clients", "mock_new_client")
def test_add(mock_clients, mock_new_client):
    with patch(
        "builtins.open", mock_open(read_data=json.dumps(mock_clients))
    ) as mock_file_open:
        clients_file = ClientModel()
        client = clients_file.add(mock_new_client)
        assert client["client_id"] in clients_file.clients
        assert "secret" in client


@pytest.mark.usefixtures("mock_clients")
def test_remove(mock_clients):
    with patch(
        "builtins.open", mock_open(read_data=json.dumps(mock_clients))
    ) as mock_file_open:
        clients_file = ClientModel()
        client = clients_file.find("client-2")
        clients_file.remove(client)
        assert len(clients_file.clients) == 1
        assert "client-2" not in clients_file.clients


@pytest.mark.usefixtures("mock_clients")
def test_update(mock_clients):
    with patch(
        "builtins.open", mock_open(read_data=json.dumps(mock_clients))
    ) as mock_file_open:
        clients_file = ClientModel()
        client = clients_file.find("client-1")
        assert client["subscription"] == "soar.#"
        client["subscription"] = "soar.client-1.#"
        clients_file.update(client)
        client = clients_file.find("client-1")
        assert len(clients_file.clients) == 2
        assert client["subscription"] == "soar.client-1.#"


@pytest.mark.usefixtures("mock_clients")
def test_save(mock_clients):
    with patch(
        "builtins.open", mock_open(read_data=json.dumps(mock_clients))
    ) as mock_file_open:
        clients_file = ClientModel()
        mock_file_open.assert_any_call("./data/clients.json", "r")
        clients_file.save()
        mock_file_open.assert_any_call("./data/clients.json", "w")
        mock_file_open().write.has_any_call(json.dumps(mock_clients))


@pytest.mark.usefixtures("mock_clients")
def test_secret(mock_clients):
    with patch(
        "builtins.open", mock_open(read_data=json.dumps(mock_clients))
    ) as mock_file_open:
        secret_pattern = "^[A-Za-z0-9]+$"
        clients_file = ClientModel()
        secret = clients_file.secret()
        assert len(secret) == 36
        assert re.match(secret_pattern, secret)
        secret = clients_file.secret(48)
        assert len(secret) == 48
