import cryptography
from datetime import datetime, timedelta
import json
import math
import os
import pytest
import re
from unittest.mock import patch, mock_open, call
from models.token_model import TokenModel


def test_init():
    token = TokenModel()
    assert type(token.fernet) == cryptography.fernet.Fernet
    assert token.token_lifetime_hours == 24


def test_get_fernet_no_env():
    token = TokenModel()
    token.getFernet()
    assert type(token.fernet) == cryptography.fernet.Fernet


@pytest.mark.usefixtures("mock_token_secret")
def test_get_fernet_env(mock_token_secret):
    with patch.dict(os.environ, {"SOAR_TOKEN_SECRET": mock_token_secret}):
        token = TokenModel()
        token.getFernet()
        assert type(token.fernet) == cryptography.fernet.Fernet
        assert token.key == mock_token_secret


def test_get_key_no_env():
    token = TokenModel()
    secret = token.getKey()
    assert secret is not None
    assert token.key is not None
    assert secret == token.key


@pytest.mark.usefixtures("mock_token_secret")
def test_get_key_env(mock_token_secret):
    with patch.dict(os.environ, {"SOAR_TOKEN_SECRET": mock_token_secret}):
        token = TokenModel()
        secret = token.getKey()
        assert secret == mock_token_secret
        assert token.key == mock_token_secret


def test_set_secret():
    with patch.dict(os.environ):
        token = TokenModel()
        token.setSecret()
        assert os.getenv(token.env_secret) == token.key


def test_get_expiry_no_env():
    token = TokenModel()
    now = datetime.utcnow()
    expected = now + timedelta(hours=token.token_lifetime_hours)
    returned = datetime.strptime(token.getExpiry(), "%Y-%m-%dT%H:%M:%S.%f")
    diff_seconds = abs(int((returned - expected).total_seconds()))
    # Although the expected and returned are calculated separately
    # the difference is generally a few microseconds.
    assert diff_seconds == 0
    lifetime = math.ceil((returned - now).total_seconds())
    assert lifetime >= 60 * 60 * 24


def test_get_expiry_env():
    with patch.dict(os.environ, {"SOAR_TOKEN_LIFETIME": "1"}):
        token = TokenModel()
        now = datetime.utcnow()
        expected = now + timedelta(hours=token.token_lifetime_hours)
        returned = datetime.fromisoformat(token.getExpiry())
        diff_seconds = abs(int((returned - expected).total_seconds()))

        assert diff_seconds == 0
        lifetime = math.ceil((returned - now).total_seconds())
        assert lifetime >= 60 * 60


def test_encrypt_decrypt_no_env():
    token = TokenModel()
    clear = "test"
    cipher = token.encrypt(clear)
    decoded = token.decrypt(cipher["token"])
    assert decoded["client_id"] == clear
    assert decoded["expiry"] == cipher["expiry"]


@pytest.mark.usefixtures("mock_token_secret")
def test_encrypt_decrypt_env(mock_token_secret):
    with patch.dict(os.environ, {"SOAR_TOKEN_SECRET": mock_token_secret}):
        token = TokenModel()
        clear = "test"
        cipher = token.encrypt(clear)
        decoded = token.decrypt(cipher["token"])
        assert decoded["client_id"] == clear
        assert decoded["expiry"] == cipher["expiry"]
        # A separate instance with the same secret
        # is able to decode existing encrypted data
        token = TokenModel()
        decoded = token.decrypt(cipher["token"])
        assert decoded["client_id"] == clear
        assert decoded["expiry"] == cipher["expiry"]


def test_get():
    # get returns an encrypted token
    token = TokenModel()
    response = token.get("test")
    assert "token" in response
    assert "expiry" in response
    decoded = token.decrypt(response["token"])
    assert "client_id" in decoded
    assert decoded["client_id"] == "test"


def test_validate():
    token = TokenModel()
    valid_response = token.get("test")
    valid_token = valid_response["token"]
    validation = token.validate(valid_token)
    assert validation["valid"]
    assert "client_id" in validation
    invalid_token = "abc" + valid_token[3:]
    validation = token.validate(invalid_token)
    assert not validation["valid"]
    assert "client_id" not in validation
